@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -36,12 +36,18 @@
 #define NV_MAX_ISR_DELAY_MS           (NV_MAX_ISR_DELAY_US / 1000)
 #define NV_NSECS_TO_JIFFIES(nsec)     ((nsec) * HZ / 1000000000)
 
-#if !defined(NV_TIMESPEC64_PRESENT)
-struct timespec64 {
-    __s64 tv_sec;
-    long  tv_nsec;
-};
+/*
+ * in_hardirq() was added in v5.11-rc1 (2020-12-15) to replace in_irq().
+ * Fall back to in_irq() for older kernels that don't have in_hardirq().
+ */
+static inline NvBool nv_in_hardirq(void)
+{
+#if defined(in_hardirq)
+    return in_hardirq();
+#else
+    return in_irq();
 #endif
+}
 
 #if !defined(NV_KTIME_GET_RAW_TS64_PRESENT)
 static inline void ktime_get_raw_ts64(struct timespec64 *ts64)
@@ -53,16 +59,6 @@ static inline void ktime_get_raw_ts64(struct timespec64 *ts64)
 }
 #endif
 
-#if !defined(NV_KTIME_GET_REAL_TS64_PRESENT)
-static inline void ktime_get_real_ts64(struct timespec64 *ts64)
-{
-    struct timeval tv;
-    do_gettimeofday(&tv);
-    ts64->tv_sec = tv.tv_sec;
-    ts64->tv_nsec = tv.tv_usec * (NvU64) NSEC_PER_USEC;
-}
-#endif
-
 static NvBool nv_timer_less_than
 (
     const struct timespec64 *a,
@@ -73,49 +69,6 @@ static NvBool nv_timer_less_than
                                     : (a->tv_sec < b->tv_sec);
 }
 
-#if !defined(NV_TIMESPEC64_PRESENT)
-static inline struct timespec64 timespec64_add
-(
-    const struct timespec64    a,
-    const struct timespec64    b
-)
-{
-    struct timespec64 result;
-
-    result.tv_sec = a.tv_sec + b.tv_sec;
-    result.tv_nsec = a.tv_nsec + b.tv_nsec;
-    while (result.tv_nsec >= NSEC_PER_SEC)
-    {
-        ++result.tv_sec;
-        result.tv_nsec -= NSEC_PER_SEC;
-    }
-    return result;
-}
-
-static inline struct timespec64  timespec64_sub
-(
-    const struct timespec64    a,
-    const struct timespec64    b
-)
-{
-    struct timespec64 result;
-
-    result.tv_sec = a.tv_sec - b.tv_sec;
-    result.tv_nsec = a.tv_nsec - b.tv_nsec;
-    while (result.tv_nsec < 0)
-    {
-        --(result.tv_sec);
-        result.tv_nsec += NSEC_PER_SEC;
-    }
-    return result;
-}
-
-static inline s64 timespec64_to_ns(struct timespec64 *ts)
-{
-    return ((s64) ts->tv_sec *  NSEC_PER_SEC) + ts->tv_nsec;
-}
-#endif
-
 static inline NvU64 nv_ktime_get_raw_ns(void)
 {
     struct timespec64 ts;
@@ -139,10 +92,10 @@ static inline NV_STATUS nv_sleep_us(unsigned int us)
 #ifdef NV_CHECK_DELAY_ACCURACY
     struct timespec64 tm1, tm2, tm_diff;
 
-    ktime_get_real_ts64(&tm1);
+    ktime_get_raw_ts64(&tm1);
 #endif
 
-    if (in_irq() && (us > NV_MAX_ISR_DELAY_US))
+    if (nv_in_hardirq() && (us > NV_MAX_ISR_DELAY_US))
         return NV_ERR_GENERIC;
 
     mdelay_safe_msec = us / 1000;
@@ -154,10 +107,9 @@ static inline NV_STATUS nv_sleep_us(unsigned int us)
         udelay(usec);
 
 #ifdef NV_CHECK_DELAY_ACCURACY
-    ktime_get_real_ts64(&tm2);
+    ktime_get_raw_ts64(&tm2);
     tm_diff = timespec64_sub(tm2, tm1);
-    nv_printf(NV_DBG_ERRORS,
-             "NVRM: delay of %d usec results in actual delay of 0x%llx nsec\n",
+    pr_info("NVRM: delay of %d usec results in actual delay of 0x%llu nsec\n",
              us, timespec64_to_ns(&tm_diff));
 #endif
     return NV_OK;
@@ -183,12 +135,12 @@ static inline NV_STATUS nv_sleep_ms(unsigned int ms)
     struct timespec64 tm_start;
 #endif
 
-    ktime_get_real_ts64(&tm_aux);
+    ktime_get_raw_ts64(&tm_aux);
 #ifdef NV_CHECK_DELAY_ACCURACY
     tm_start = tm_aux;
 #endif
 
-    if (in_irq() && (ms > NV_MAX_ISR_DELAY_MS))
+    if (nv_in_hardirq() && (ms > NV_MAX_ISR_DELAY_MS))
     {
         return NV_ERR_GENERIC;
     }
@@ -219,7 +171,7 @@ static inline NV_STATUS nv_sleep_ms(unsigned int ms)
         do
         {
             schedule_timeout(jiffies);
-            ktime_get_real_ts64(&tm_aux);
+            ktime_get_raw_ts64(&tm_aux);
             if (nv_timer_less_than(&tm_aux, &tm_end))
             {
                 tm_aux = timespec64_sub(tm_end, tm_aux);
@@ -241,10 +193,9 @@ static inline NV_STATUS nv_sleep_ms(unsigned int ms)
         ndelay(ns);
     }
 #ifdef NV_CHECK_DELAY_ACCURACY
-    ktime_get_real_ts64(&tm_aux);
+    ktime_get_raw_ts64(&tm_aux);
     tm_aux = timespec64_sub(tm_aux, tm_start);
-    nv_printf(NV_DBG_ERRORS,
-             "NVRM: delay of %d msec results in actual delay of %d.%09d sec\n",
+    pr_info("NVRM: delay of %d msec results in actual delay of %lld.%09ld sec\n",
              ms, tm_aux.tv_sec, tm_aux.tv_nsec);
 #endif
     return NV_OK;
