@@ -22,23 +22,19 @@
 
 #include "nvidia-drm-conftest.h"
 
-#if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)
+#if defined(NV_DRM_AVAILABLE)
 
 #include "nvidia-drm-gem-nvkms-memory.h"
 #include "nvidia-drm-helper.h"
 #include "nvidia-drm-ioctl.h"
 
-#if defined(NV_DRM_DRM_DRV_H_PRESENT)
 #include <drm/drm_drv.h>
-#endif
-
-#if defined(NV_DRM_DRM_PRIME_H_PRESENT)
 #include <drm/drm_prime.h>
-#endif
 
 #include <linux/io.h>
-
-#include "nv-mm.h"
+#if defined(NV_BSD)
+#include <vm/vm_pageout.h>
+#endif
 
 static void __nv_drm_gem_nvkms_memory_free(struct nv_drm_gem_object *nv_gem)
 {
@@ -57,6 +53,10 @@ static void __nv_drm_gem_nvkms_memory_free(struct nv_drm_gem_object *nv_gem)
                            nv_nvkms_memory->pPhysicalAddress);
     }
 
+    if (nv_nvkms_memory->pages_count != 0) {
+        nvKms->freeMemoryPages((NvU64 *)nv_nvkms_memory->pages);
+    }
+
     /* Free NvKmsKapiMemory handle associated with this gem object */
 
     nvKms->freeMemory(nv_dev->pDevice, nv_nvkms_memory->base.pMemory);
@@ -64,9 +64,20 @@ static void __nv_drm_gem_nvkms_memory_free(struct nv_drm_gem_object *nv_gem)
     nv_drm_free(nv_nvkms_memory);
 }
 
+static int __nv_drm_gem_nvkms_map(
+    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory);
+
 static int __nv_drm_gem_nvkms_mmap(struct nv_drm_gem_object *nv_gem,
                                    struct vm_area_struct *vma)
 {
+    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory =
+        to_nv_nvkms_memory(nv_gem);
+
+    int ret = __nv_drm_gem_nvkms_map(nv_nvkms_memory);
+    if (ret) {
+       return ret;
+    }
+
     return drm_gem_mmap_obj(&nv_gem->base,
                 drm_vma_node_size(&nv_gem->base.vma_node) << PAGE_SHIFT, vma);
 }
@@ -76,24 +87,38 @@ static vm_fault_t __nv_drm_gem_nvkms_handle_vma_fault(
     struct vm_area_struct *vma,
     struct vm_fault *vmf)
 {
-#if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)
     struct nv_drm_gem_nvkms_memory *nv_nvkms_memory =
         to_nv_nvkms_memory(nv_gem);
-    unsigned long address = nv_page_fault_va(vmf);
+    unsigned long address = vmf->address;
     struct drm_gem_object *gem = vma->vm_private_data;
     unsigned long page_offset, pfn;
     vm_fault_t ret;
 
-    pfn = (unsigned long)(uintptr_t)nv_nvkms_memory->pPhysicalAddress;
-    pfn >>= PAGE_SHIFT;
-
     page_offset = vmf->pgoff - drm_vma_node_start(&gem->vma_node);
 
-#if defined(NV_VMF_INSERT_PFN_PRESENT)
-    ret = vmf_insert_pfn(vma, address, pfn + page_offset);
-#else
-    ret = vm_insert_pfn(vma, address, pfn + page_offset);
+    if (nv_nvkms_memory->pages_count == 0) {
+        pfn = (unsigned long)(uintptr_t)nv_nvkms_memory->pPhysicalAddress;
+        pfn >>= PAGE_SHIFT;
+#if defined(NV_LINUX)
+        /*
+         * FreeBSD doesn't set pgoff. We instead have pfn be the base physical
+         * address, and we will calculate the index pidx from the virtual address.
+         *
+         * This only works because linux_cdev_pager_populate passes the pidx as
+         * vmf->virtual_address. Then we turn the virtual address
+         * into a physical page number.
+         */
+        pfn += page_offset;
+#endif
+    } else {
+        BUG_ON(page_offset >= nv_nvkms_memory->pages_count);
+        pfn = page_to_pfn(nv_nvkms_memory->pages[page_offset]);
+    }
 
+#if defined(NV_VMF_INSERT_PFN_PRESENT)
+    ret = vmf_insert_pfn(vma, address, pfn);
+#else
+    ret = vm_insert_pfn(vma, address, pfn);
     switch (ret) {
         case 0:
         case -EBUSY:
@@ -113,8 +138,6 @@ static vm_fault_t __nv_drm_gem_nvkms_handle_vma_fault(
     }
 #endif /* defined(NV_VMF_INSERT_PFN_PRESENT) */
     return ret;
-#endif /* defined(NV_DRM_ATOMIC_MODESET_AVAILABLE) */
-    return VM_FAULT_SIGBUS;
 }
 
 static struct drm_gem_object *__nv_drm_gem_nvkms_prime_dup(
@@ -122,11 +145,22 @@ static struct drm_gem_object *__nv_drm_gem_nvkms_prime_dup(
     const struct nv_drm_gem_object *nv_gem_src);
 
 static int __nv_drm_gem_nvkms_map(
-    struct nv_drm_device *nv_dev,
-    struct NvKmsKapiMemory *pMemory,
-    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory,
-    uint64_t size)
+    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory)
 {
+    int ret = 0;
+    struct nv_drm_device *nv_dev = nv_nvkms_memory->base.nv_dev;
+    struct NvKmsKapiMemory *pMemory = nv_nvkms_memory->base.pMemory;
+
+    mutex_lock(&nv_nvkms_memory->map_lock);
+
+    if (nv_nvkms_memory->physically_mapped) {
+        goto done;
+    }
+
+    if (!nvKms->isVidmem(pMemory)) {
+        goto done;
+    }
+
     if (!nvKms->mapMemory(nv_dev->pDevice,
                           pMemory,
                           NVKMS_KAPI_MAPPING_TYPE_USER,
@@ -135,25 +169,68 @@ static int __nv_drm_gem_nvkms_map(
             nv_dev,
             "Failed to map NvKmsKapiMemory 0x%p",
             pMemory);
-        return -ENOMEM;
+        ret = -ENOMEM;
+        goto done;
     }
 
-    if (nv_dev->hasVideoMemory) {
-        nv_nvkms_memory->pWriteCombinedIORemapAddress = ioremap_wc(
+    nv_nvkms_memory->pWriteCombinedIORemapAddress = ioremap_wc(
             (uintptr_t)nv_nvkms_memory->pPhysicalAddress,
-            size);
+            nv_nvkms_memory->base.base.size);
 
-        if (!nv_nvkms_memory->pWriteCombinedIORemapAddress) {
-            NV_DRM_DEV_LOG_INFO(
-                nv_dev,
-                "Failed to ioremap_wc NvKmsKapiMemory 0x%p",
-                pMemory);
-        }
+    if (!nv_nvkms_memory->pWriteCombinedIORemapAddress) {
+        NV_DRM_DEV_LOG_INFO(
+            nv_dev,
+            "Failed to ioremap_wc NvKmsKapiMemory 0x%p",
+            pMemory);
     }
 
     nv_nvkms_memory->physically_mapped = true;
 
-    return 0;
+done:
+    mutex_unlock(&nv_nvkms_memory->map_lock);
+    return ret;
+}
+
+static void *__nv_drm_gem_nvkms_prime_vmap(
+    struct nv_drm_gem_object *nv_gem)
+{
+    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory =
+        to_nv_nvkms_memory(nv_gem);
+
+    int ret = __nv_drm_gem_nvkms_map(nv_nvkms_memory);
+    if (ret) {
+       return ERR_PTR(ret);
+    }
+
+    if (nv_nvkms_memory->physically_mapped) {
+        return nv_nvkms_memory->pWriteCombinedIORemapAddress;
+    }
+
+    /*
+     * If this buffer isn't physically mapped, it might be backed by struct
+     * pages. Use vmap in that case. Do a noncached mapping for system memory
+     * as display is non io-coherent device in case of Tegra.
+     */
+    if (nv_nvkms_memory->pages_count > 0) {
+         return nv_drm_vmap(nv_nvkms_memory->pages,
+                            nv_nvkms_memory->pages_count,
+                            false);
+    }
+
+    return ERR_PTR(-ENOMEM);
+}
+
+static void __nv_drm_gem_nvkms_prime_vunmap(
+    struct nv_drm_gem_object *nv_gem,
+    void *address)
+{
+    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory =
+        to_nv_nvkms_memory(nv_gem);
+
+    if (!nv_nvkms_memory->physically_mapped &&
+        nv_nvkms_memory->pages_count > 0) {
+        nv_drm_vunmap(address);
+    }
 }
 
 static int __nv_drm_gem_map_nvkms_memory_offset(
@@ -161,45 +238,28 @@ static int __nv_drm_gem_map_nvkms_memory_offset(
     struct nv_drm_gem_object *nv_gem,
     uint64_t *offset)
 {
-    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory =
-        to_nv_nvkms_memory(nv_gem);
-
-    if (!nv_nvkms_memory->physically_mapped) {
-        int ret = __nv_drm_gem_nvkms_map(nv_dev,
-                                         nv_nvkms_memory->base.pMemory,
-                                         nv_nvkms_memory,
-                                         nv_nvkms_memory->base.base.size);
-        if (ret) {
-           return ret;
-        }
-    }
-
-    return nv_drm_gem_create_mmap_offset(&nv_nvkms_memory->base, offset);
+    return nv_drm_gem_create_mmap_offset(nv_gem, offset);
 }
 
 static struct sg_table *__nv_drm_gem_nvkms_memory_prime_get_sg_table(
     struct nv_drm_gem_object *nv_gem)
 {
     struct nv_drm_device *nv_dev = nv_gem->nv_dev;
+    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory =
+        to_nv_nvkms_memory(nv_gem);
     struct sg_table *sg_table;
-    NvU64 *pages;
-    NvU32 numPages;
 
-    if (!nvKms->getMemoryPages(nv_dev->pDevice,
-                               nv_gem->pMemory,
-                               &pages,
-                               &numPages)) {
-        NV_DRM_DEV_LOG_ERR(
+    if (nv_nvkms_memory->pages_count == 0) {
+        NV_DRM_DEV_DEBUG_DRIVER(
                 nv_dev,
-                "Failed to get memory pages for NvKmsKapiMemory 0x%p",
+                "Cannot create sg_table for NvKmsKapiMemory 0x%p",
                 nv_gem->pMemory);
-        return NULL;
+        return ERR_PTR(-ENOMEM);
     }
 
     sg_table = nv_drm_prime_pages_to_sg(nv_dev->dev,
-                                        (struct page **)pages, numPages);
-
-    nvKms->freeMemoryPages(pages);
+                                        nv_nvkms_memory->pages,
+                                        nv_nvkms_memory->pages_count);
 
     return sg_table;
 }
@@ -207,12 +267,61 @@ static struct sg_table *__nv_drm_gem_nvkms_memory_prime_get_sg_table(
 const struct nv_drm_gem_object_funcs nv_gem_nvkms_memory_ops = {
     .free = __nv_drm_gem_nvkms_memory_free,
     .prime_dup = __nv_drm_gem_nvkms_prime_dup,
+    .prime_vmap = __nv_drm_gem_nvkms_prime_vmap,
+    .prime_vunmap = __nv_drm_gem_nvkms_prime_vunmap,
     .mmap = __nv_drm_gem_nvkms_mmap,
     .handle_vma_fault = __nv_drm_gem_nvkms_handle_vma_fault,
     .create_mmap_offset = __nv_drm_gem_map_nvkms_memory_offset,
     .prime_get_sg_table = __nv_drm_gem_nvkms_memory_prime_get_sg_table,
 };
 
+static int __nv_drm_nvkms_gem_obj_init(
+    struct nv_drm_device *nv_dev,
+    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory,
+    struct NvKmsKapiMemory *pMemory,
+    uint64_t size)
+{
+    NvU64 *pages = NULL;
+    NvU32 numPages = 0;
+
+    if ((size % PAGE_SIZE) != 0) {
+        NV_DRM_DEV_LOG_ERR(
+            nv_dev,
+            "NvKmsKapiMemory 0x%p size should be in a multiple of page size to "
+            "create a gem object",
+            pMemory);
+        return -EINVAL;
+    }
+
+    mutex_init(&nv_nvkms_memory->map_lock);
+    nv_nvkms_memory->pPhysicalAddress = NULL;
+    nv_nvkms_memory->pWriteCombinedIORemapAddress = NULL;
+    nv_nvkms_memory->physically_mapped = false;
+
+    if (!nvKms->isVidmem(pMemory) &&
+        !nvKms->getMemoryPages(nv_dev->pDevice,
+                               pMemory,
+                               &pages,
+                               &numPages)) {
+        /* GetMemoryPages will fail for vidmem allocations,
+         * but it should not fail for sysmem allocations. */
+        NV_DRM_DEV_LOG_ERR(nv_dev,
+            "Failed to get memory pages for NvKmsKapiMemory 0x%p",
+            pMemory);
+        return -ENOMEM;
+    }
+    nv_nvkms_memory->pages_count = numPages;
+    nv_nvkms_memory->pages = (struct page **)pages;
+
+    nv_drm_gem_object_init(nv_dev,
+                           &nv_nvkms_memory->base,
+                           &nv_gem_nvkms_memory_ops,
+                           size,
+                           pMemory);
+
+    return 0;
+}
+
 int nv_drm_dumb_create(
     struct drm_file *file_priv,
     struct drm_device *dev, struct drm_mode_create_dumb *args)
@@ -221,6 +330,7 @@ int nv_drm_dumb_create(
     struct nv_drm_gem_nvkms_memory *nv_nvkms_memory;
     uint8_t compressible = 0;
     struct NvKmsKapiMemory *pMemory;
+    struct NvKmsKapiAllocateMemoryParams allocParams = { };
     int ret = 0;
 
     args->pitch = roundup(args->width * ((args->bpp + 7) >> 3),
@@ -238,48 +348,43 @@ int nv_drm_dumb_create(
         goto fail;
     }
 
-    if (nv_dev->hasVideoMemory) {
-        pMemory = nvKms->allocateVideoMemory(nv_dev->pDevice,
-                                             NvKmsSurfaceMemoryLayoutPitch,
-                                             args->size,
-                                             &compressible);
-    } else {
-        pMemory = nvKms->allocateSystemMemory(nv_dev->pDevice,
-                                              NvKmsSurfaceMemoryLayoutPitch,
-                                              args->size,
-                                              &compressible);
-    }
+    allocParams.layout = NvKmsSurfaceMemoryLayoutPitch;
+    allocParams.type = NVKMS_KAPI_ALLOCATION_TYPE_SCANOUT;
+    allocParams.size = args->size;
+    allocParams.noDisplayCaching = true;
+    allocParams.useVideoMemory = nv_dev->hasVideoMemory;
+    allocParams.compressible = &compressible;
 
+    pMemory = nvKms->allocateMemory(nv_dev->pDevice, &allocParams);
     if (pMemory == NULL) {
         ret = -ENOMEM;
         NV_DRM_DEV_LOG_ERR(
             nv_dev,
-            "Failed to allocate NvKmsKapiMemory for dumb object of size %llu",
+            "Failed to allocate NvKmsKapiMemory for dumb object of size %" NvU64_fmtu,
             args->size);
         goto nvkms_alloc_memory_failed;
     }
 
+    ret = __nv_drm_nvkms_gem_obj_init(nv_dev, nv_nvkms_memory, pMemory, args->size);
+    if (ret) {
+        goto nvkms_gem_obj_init_failed;
+    }
+
     /* Always map dumb buffer memory up front.  Clients are only expected
      * to use dumb buffers for software rendering, so they're not much use
      * without a CPU mapping.
      */
-    ret = __nv_drm_gem_nvkms_map(nv_dev, pMemory, nv_nvkms_memory, args->size);
+    ret = __nv_drm_gem_nvkms_map(nv_nvkms_memory);
     if (ret) {
-        goto nvkms_map_memory_failed;
+        nv_drm_gem_object_unreference_unlocked(&nv_nvkms_memory->base);
+        goto fail;
     }
 
-    nv_drm_gem_object_init(nv_dev,
-                           &nv_nvkms_memory->base,
-                           &nv_gem_nvkms_memory_ops,
-                           args->size,
-                           pMemory);
-
     return nv_drm_gem_handle_create_drop_reference(file_priv,
                                                    &nv_nvkms_memory->base,
                                                    &args->handle);
 
-nvkms_map_memory_failed:
-
+nvkms_gem_obj_init_failed:
     nvKms->freeMemory(nv_dev->pDevice, pMemory);
 
 nvkms_alloc_memory_failed:
@@ -289,23 +394,6 @@ fail:
     return ret;
 }
 
-static void __nv_drm_nvkms_gem_obj_init(
-    struct nv_drm_device *nv_dev,
-    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory,
-    struct NvKmsKapiMemory *pMemory,
-    uint64_t size)
-{
-    nv_nvkms_memory->pPhysicalAddress = NULL;
-    nv_nvkms_memory->pWriteCombinedIORemapAddress = NULL;
-    nv_nvkms_memory->physically_mapped = false;
-
-    nv_drm_gem_object_init(nv_dev,
-                           &nv_nvkms_memory->base,
-                           &nv_gem_nvkms_memory_ops,
-                           size,
-                           pMemory);
-}
-
 int nv_drm_gem_import_nvkms_memory_ioctl(struct drm_device *dev,
                                          void *data, struct drm_file *filep)
 {
@@ -316,7 +404,7 @@ int nv_drm_gem_import_nvkms_memory_ioctl(struct drm_device *dev,
     int ret;
 
     if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
-        ret = -EINVAL;
+        ret = -EOPNOTSUPP;
         goto failed;
     }
 
@@ -339,11 +427,16 @@ int nv_drm_gem_import_nvkms_memory_ioctl(struct drm_device *dev,
         goto nvkms_import_memory_failed;
     }
 
-    __nv_drm_nvkms_gem_obj_init(nv_dev, nv_nvkms_memory, pMemory, p->mem_size);
+    ret = __nv_drm_nvkms_gem_obj_init(nv_dev, nv_nvkms_memory, pMemory, p->mem_size);
+    if (ret) {
+        goto nvkms_gem_obj_init_failed;
+    }
 
     return nv_drm_gem_handle_create_drop_reference(filep,
                                                    &nv_nvkms_memory->base,
                                                    &p->handle);
+nvkms_gem_obj_init_failed:
+    nvKms->freeMemory(nv_dev->pDevice, pMemory);
 
 nvkms_import_memory_failed:
     nv_drm_free(nv_nvkms_memory);
@@ -361,7 +454,7 @@ int nv_drm_gem_export_nvkms_memory_ioctl(struct drm_device *dev,
     int ret = 0;
 
     if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
-        ret = -EINVAL;
+        ret = -EOPNOTSUPP;
         goto done;
     }
 
@@ -372,7 +465,6 @@ int nv_drm_gem_export_nvkms_memory_ioctl(struct drm_device *dev,
     }
 
     if ((nv_nvkms_memory = nv_drm_gem_object_nvkms_memory_lookup(
-                    dev,
                     filep,
                     p->handle)) == NULL) {
         ret = -EINVAL;
@@ -409,15 +501,16 @@ int nv_drm_gem_alloc_nvkms_memory_ioctl(struct drm_device *dev,
     struct drm_nvidia_gem_alloc_nvkms_memory_params *p = data;
     struct nv_drm_gem_nvkms_memory *nv_nvkms_memory = NULL;
     struct NvKmsKapiMemory *pMemory;
-    enum NvKmsSurfaceMemoryLayout layout;
+    struct NvKmsKapiAllocateMemoryParams allocParams = { };
     int ret = 0;
 
     if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
-        ret = -EINVAL;
+        ret = -EOPNOTSUPP;
         goto failed;
     }
 
-    if (p->__pad != 0) {
+    if ((p->__pad0 != 0) || (p->__pad1 != 0)) {
+        ret = -EINVAL;
         NV_DRM_DEV_LOG_ERR(nv_dev, "non-zero value in padding field");
         goto failed;
     }
@@ -428,21 +521,15 @@ int nv_drm_gem_alloc_nvkms_memory_ioctl(struct drm_device *dev,
         goto failed;
     }
 
-    layout = p->block_linear ?
+    allocParams.layout = p->block_linear ?
         NvKmsSurfaceMemoryLayoutBlockLinear : NvKmsSurfaceMemoryLayoutPitch;
+    allocParams.type = (p->flags & NV_GEM_ALLOC_NO_SCANOUT) ?
+        NVKMS_KAPI_ALLOCATION_TYPE_OFFSCREEN : NVKMS_KAPI_ALLOCATION_TYPE_SCANOUT;
+    allocParams.size = p->memory_size;
+    allocParams.useVideoMemory = nv_dev->hasVideoMemory;
+    allocParams.compressible = &p->compressible;
 
-    if (nv_dev->hasVideoMemory) {
-        pMemory = nvKms->allocateVideoMemory(nv_dev->pDevice,
-                                             layout,
-                                             p->memory_size,
-                                             &p->compressible);
-    } else {
-        pMemory = nvKms->allocateSystemMemory(nv_dev->pDevice,
-                                              layout,
-                                              p->memory_size,
-                                              &p->compressible);
-    }
-
+    pMemory = nvKms->allocateMemory(nv_dev->pDevice, &allocParams);
     if (pMemory == NULL) {
         ret = -EINVAL;
         NV_DRM_DEV_LOG_ERR(nv_dev,
@@ -450,12 +537,19 @@ int nv_drm_gem_alloc_nvkms_memory_ioctl(struct drm_device *dev,
         goto nvkms_alloc_memory_failed;
     }
 
-    __nv_drm_nvkms_gem_obj_init(nv_dev, nv_nvkms_memory, pMemory,
-                                p->memory_size);
+    ret = __nv_drm_nvkms_gem_obj_init(nv_dev, nv_nvkms_memory, pMemory,
+                                      p->memory_size);
+    if (ret) {
+        goto nvkms_gem_obj_init_failed;
+    }
 
     return nv_drm_gem_handle_create_drop_reference(filep,
                                                    &nv_nvkms_memory->base,
                                                    &p->handle);
+
+nvkms_gem_obj_init_failed:
+    nvKms->freeMemory(nv_dev->pDevice, pMemory);
+
 nvkms_alloc_memory_failed:
     nv_drm_free(nv_nvkms_memory);
 
@@ -469,14 +563,12 @@ static struct drm_gem_object *__nv_drm_gem_nvkms_prime_dup(
 {
     struct nv_drm_device *nv_dev = to_nv_device(dev);
     const struct nv_drm_device *nv_dev_src;
-    const struct nv_drm_gem_nvkms_memory *nv_nvkms_memory_src;
     struct nv_drm_gem_nvkms_memory *nv_nvkms_memory;
     struct NvKmsKapiMemory *pMemory;
 
     BUG_ON(nv_gem_src == NULL || nv_gem_src->ops != &nv_gem_nvkms_memory_ops);
 
     nv_dev_src = to_nv_device(nv_gem_src->base.dev);
-    nv_nvkms_memory_src = to_nv_nvkms_memory_const(nv_gem_src);
 
     if ((nv_nvkms_memory =
             nv_drm_calloc(1, sizeof(*nv_nvkms_memory))) == NULL) {
@@ -492,13 +584,18 @@ static struct drm_gem_object *__nv_drm_gem_nvkms_prime_dup(
         goto nvkms_dup_memory_failed;
     }
 
-    __nv_drm_nvkms_gem_obj_init(nv_dev,
-                                nv_nvkms_memory,
-                                pMemory,
-                                nv_gem_src->base.size);
+    if (__nv_drm_nvkms_gem_obj_init(nv_dev,
+                                    nv_nvkms_memory,
+                                    pMemory,
+                                    nv_gem_src->base.size)) {
+        goto nvkms_gem_obj_init_failed;
+    }
 
     return &nv_nvkms_memory->base.base;
 
+nvkms_gem_obj_init_failed:
+    nvKms->freeMemory(nv_dev->pDevice, pMemory);
+
 nvkms_dup_memory_failed:
     nv_drm_free(nv_nvkms_memory);
 
@@ -514,7 +611,6 @@ int nv_drm_dumb_map_offset(struct drm_file *file,
     int ret = -EINVAL;
 
     if ((nv_nvkms_memory = nv_drm_gem_object_nvkms_memory_lookup(
-                    dev,
                     file,
                     handle)) == NULL) {
         NV_DRM_DEV_LOG_ERR(
@@ -532,11 +628,13 @@ int nv_drm_dumb_map_offset(struct drm_file *file,
     return ret;
 }
 
+#if defined(NV_DRM_DRIVER_HAS_DUMB_DESTROY)
 int nv_drm_dumb_destroy(struct drm_file *file,
                         struct drm_device *dev,
                         uint32_t handle)
 {
     return drm_gem_handle_delete(file, handle);
 }
+#endif /* NV_DRM_DRIVER_HAS_DUMB_DESTROY */
 
 #endif
