@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2019 NVIDIA Corporation
+    Copyright (c) 2016-2024 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -30,9 +30,9 @@
 #include "uvm_rb_tree.h"
 #include "nv-kref.h"
 
-// This structure contains the VA spaces of all the subcontexts in a TSG. It
+// This structure contains the GPU VA spaces of all the subcontexts in a TSG. It
 // is stored in a per-GPU UVM RB tree and is required to perform instance_ptr
-// to VA space translations when channels are registered in a subcontext,
+// to GPU VA space translations when channels are registered in a subcontext,
 // since SM fault/access counter notification packets may report any
 // instance_ptr in the TSG.
 typedef struct
@@ -46,7 +46,7 @@ typedef struct
     // Array of per-subcontext information
     struct
     {
-        uvm_va_space_t *va_space;
+        uvm_gpu_va_space_t *gpu_va_space;
 
         // Number of instance pointers referencing this specific subcontext
         NvU32 refcount;
@@ -98,17 +98,11 @@ struct uvm_user_channel_struct
         // If valid is true, tsg_id contains the ID of the TSG
         NvU32 id;
 
-        // If valid is true, this is the maximum number of subcontexts in the TSG
+        // If valid is true, this is the maximum number of subcontexts in the
+        // TSG
         NvU32 max_subctx_count;
     } tsg;
 
-    // This is the value that needs to be used when ringing the channel's
-    // doorbell
-    NvU32 work_submission_token;
-
-    // This is the address of the channel's doorbell
-    volatile NvU32 *work_submission_offset;
-
     // On Turing+, the CLEAR_FAULTED method requires passing a RM-provided
     // handle to identify the channel.
     NvU32 clear_faulted_token;
@@ -117,9 +111,11 @@ struct uvm_user_channel_struct
     // channel removal
     uvm_tracker_t clear_faulted_tracker;
 
-    // Address of the NV_CHRAM_CHANNEL register. Only valid on GPUs with
-    // non_replayable_faults_supported && !has_clear_faulted_channel_method
+    // Address of the NV_CHRAM_CHANNEL register and the runlist PRI base
+    // register. Only valid on GPUs with
+    // non_replayable_faults_supported && !has_clear_faulted_channel_method.
     volatile NvU32 *chram_channel_register;
+    volatile NvU32 *runlist_pri_base_register;
 
     // Id of the SMC engine this channel is bound to, or zero if the GPU
     // does not support SMC or it is a CE channel
@@ -135,28 +131,26 @@ struct uvm_user_channel_struct
 
     // If in_subctx is true, subctx_info will point at a per-TSG data structure
     // that contains the VA spaces of all the subcontexts in the TSG. This value
-    // is assigned in uvm_gpu_add_user_channel.
+    // is assigned in uvm_parent_gpu_add_user_channel.
     uvm_user_channel_subctx_info_t *subctx_info;
 
     // Number of resources reported by RM. This is the size of both the
-    // resources and va_ranges arrays.
+    // resources and channel_ranges arrays.
     size_t num_resources;
 
-    // Array of all resources for this channel, both global (physically shared
-    // across all TSGs) and local (physically shared across channels in a TSG).
-    // The distinction is ignored by the UVM driver, which shares virtual
-    // mappings for these physical resources across all channels in the same GPU
-    // VA space and TSG. Each channel will retain the mappings (VA ranges) it
-    // uses at channel register and will release them at
+    // Array of all resources for this channel, shared or not. Virtual mappings
+    // for matching physical resources are shared across all channels in the
+    // same GPU VA space and TSG. Each channel will retain the mappings (VA
+    // ranges) it uses at channel register and will release them at
     // uvm_user_channel_destroy_detached, so these physical resources outlive
     // the corresponding VA ranges.
     UvmGpuChannelResourceInfo *resources;
 
-    // Array of all VA ranges associated with this channel. Entry i in this
+    // Array of all channel ranges associated with this channel. Entry i in this
     // array corresponds to resource i in the resources array above and has the
     // same descriptor. uvm_user_channel_detach will drop the ref counts for
-    // these VA ranges, potentially destroying them.
-    uvm_va_range_t **va_ranges;
+    // these channel ranges, potentially destroying them.
+    uvm_va_range_channel_t **channel_ranges;
 
     // Physical instance pointer. There is a 1:1 mapping between instance
     // pointer and channel. GPU faults report an instance pointer, and the GPU
@@ -178,7 +172,8 @@ struct uvm_user_channel_struct
     // Opaque object which refers to this channel in the nvUvmInterface APIs
     void *rm_retained_channel;
 
-    // Hardware channel ID. This is used for debugging and fault processing.
+    // Hardware runlist and channel IDs, used for debugging and fault processing
+    NvU32 hw_runlist_id;
     NvU32 hw_channel_id;
 
     // Node in the owning gpu_va_space's registered_channels list. Cleared once
